# frozen_string_literal: true

require 'awesome_print'

require 'article_fixture_gen/config'

# Generate blog post/article fixture data, with embedded marker tag pairs.
module ArticleFixtureGen
  # Command-line-specific classes.
  module CLI
    # Report contents of configuration data.
    class DumpConfig
      # Reek doesn't like :reek:BooleanParameter. Tough.
      def self.call(config:, use_colour: false)
        DumpConfig.new(config).call use_colour
      end

      def call(use_colour)
        @use_colour = use_colour
        report
      end

      protected

      def initialize(config)
        @config = config.to_hash
        self
      end

      private

      attr_reader :config, :use_colour

      def dump_options
        { plain: !use_colour, sort_keys: true }
      end

      def report
        config.ai dump_options
      end
    end # class ArticleFixtureGen::CLI::DumpConfig
  end
end
