# frozen_string_literal: true

require_relative './mtp_decorated_markup'
require_relative './pmtp_decorator_params'

module ArticleFixtureGen
  module Data
    # Generate a copy of the specified base content with single marker-tag pairs
    # inserted before random words in the content.
    #
    # Parameters:
    # `base_markup`: *Must* be a string representing a single HTML element with
    #                child nodes, some containing text strings. The canonical
    #                example is article-body markup presented as a containing
    #                `div` with paragraphs and so on;
    # `config`:      A configuration-information object, which *must* respond to
    #                the `:pmtp_text` message with a string, and *must* respond
    #                to the `:pmtp_count` message with a non-negative integer.
    #
    class PmtpDecoratedMarkup
      def self.call(base_markup:, config:, param_builder: PmtpDecoratorParams)
        params = param_builder.call(base_markup: base_markup, config: config)
        MtpDecoratedMarkup.call(params)
      end
    end # class ArticleFixtureGen::Data::PmtpDecoratedMarkup
  end
end
