# frozen_string_literal: true

module ArticleFixtureGen
  module Data
    # Encapsulates a "single marker-tag pair", or sMTP; an HTML anchor tag with
    # a specifically-formatted ID attribute, an empty :href attribute, and no
    # enclosed content. Different than a "paired marker-tag pair", which is
    # analogous to two slightly differently-formatted sMTPs separated by content
    # (and with no overlapping pMTPs).
    class SingleMarkerTagPair
      def initialize(config:, counter:)
        @counter = counter
        @label = config.smtp_text
        IceNine.deep_freeze self
      end

      def to_html
        ['<a href="" id="', '"></a>'].join html_id
      end
      alias_method :to_s, :to_html

      private

      attr_reader :counter, :label

      def html_id
        [label, counter.to_s].join '-'
      end
    end # class ArticleFixtureGen::Data::SingleMarkerTagPair
  end
end
