# frozen_string_literal: true

require 'yaml'

module ArticleFixtureGen
  module Exe
    # Save configuration as YAML to file specified *in* configuration.
    class GenerateConfig
      def self.call(config)
        new(config).call
      end

      def call
        return unless filename
        write_output
        data.freeze
      end

      protected

      def initialize(config)
        @data = Internals.filter_data(config)
        @filename = config.generate_config.to_s
        self
      end

      private

      attr_reader :data, :filename

      OMITTED_KEYS = [:config, :config_given, :generate_config,
                      :generate_config_given].freeze
      private_constant :OMITTED_KEYS

      def write_output
        # Using a File.open block raises an IOError within Psych (YAML internal)
        outf = File.open(filename, 'w')
        YAML.dump data, outf
        outf.close
      end

      # Stateless methods
      module Internals
        def self.filter_data(config)
          config.to_hash.reject { |attrib, _| OMITTED_KEYS.include? attrib }
        end
      end
      private_constant :Internals
    end # class ArticleFixtureGen::Exe::GenerateConfig
  end
end
